/**
 * Copyright Copyright 2008-14 Simon Andrews
 *
 *    This file is part of FocalPoint.
 *
 *    FocalPoint is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FocalPoint is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package uk.ac.bbsrc.babraham.FocalPoint.Preferences;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class PrefsDialog extends JDialog implements ActionListener {

	private Preferences prefs;

	private JCheckBox showOnlyImages;
	private JCheckBox deferLoading;
	private JTextField iconWidth;
	private JTextField imageJPath;
	private JButton findIJPath;
	
	public PrefsDialog () {
		
		setTitle("Preferences...");
		getContentPane().setLayout(new BorderLayout());
		
		prefs = Preferences.getInstance();
		
		JPanel prefsPanel = new JPanel();
		JPanel buttonPanel = new JPanel();
		
		JButton saveButton = new JButton("Save and Close");
		saveButton.setActionCommand("save");
		saveButton.addActionListener(this);
		buttonPanel.add(saveButton);
		
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		// Set up the prefs panel
		prefsPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.gridx=0;
		c.gridy=0;
		c.weightx=0.5;
		c.weighty=0.5;
		c.fill=GridBagConstraints.NONE;
		c.insets = new Insets(2,5,2,5);
		c.anchor = GridBagConstraints.WEST;
		
		prefsPanel.add(new JLabel("Show only image files"),c);
		c.gridx=1;
		showOnlyImages = new JCheckBox();
		if (prefs.showOnlyImages()) {
			showOnlyImages.setSelected(true);
		}
		else {
			showOnlyImages.setSelected(false);
		}
		prefsPanel.add(showOnlyImages,c);
		
		
		c.gridx=0;
		c.gridy++;

		prefsPanel.add(new JLabel("Defer Loading Large Images"),c);
		c.gridx=1;

		deferLoading = new JCheckBox();
		if (prefs.deferLargeImages()) {
			deferLoading.setSelected(true);
		}
		else {
			deferLoading.setSelected(false);
		}
		prefsPanel.add(deferLoading,c);
		
		c.gridx=0;
		c.gridy++;

		prefsPanel.add(new JLabel("Default Icon Width (px)"),c);
		c.gridx=1;
		
		iconWidth = new JTextField(""+prefs.iconWidth(),4);
		prefsPanel.add(iconWidth,c);
		
		getContentPane().add(prefsPanel,BorderLayout.CENTER);
		
		c.gridx=0;
		c.gridy++;

		prefsPanel.add(new JLabel("ImageJ location (blank for default)"),c);
		c.gridx=1;
		
		JPanel ijPanel = new JPanel();
		ijPanel.setLayout(new BorderLayout());
		
		imageJPath = new JTextField(15);
		if (prefs.pathToImageJ() != null) {
			imageJPath.setText(prefs.pathToImageJ().getAbsolutePath());
		}
		imageJPath.setEditable(false);

		ijPanel.add(imageJPath,BorderLayout.CENTER);
		
		findIJPath = new JButton("Find");
		findIJPath.setActionCommand("find");
		findIJPath.addActionListener(this);
		
		ijPanel.add(findIJPath,BorderLayout.EAST);
		
		prefsPanel.add(ijPanel,c);
		setSize(400,200);
		setLocationRelativeTo(null);
		setVisible(true);
	}
	
	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("save")) {
			
			int width = 150;
			
			try {
				width = Integer.parseInt(iconWidth.getText());
			}
			catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(this, "Icon width wasn't an integer", "Error", JOptionPane.ERROR_MESSAGE);				
			}
			
			File f = null;
			
			if (imageJPath.getText().length() > 0) {
				f = new File(imageJPath.getText());
			}
			
			// Now update the preferences
			
			try {
				prefs.setPathToImageJ(f);
			} 
			catch (IOException ioe) {
				JOptionPane.showMessageDialog(this, "Error setting ImageJ location:"+ioe.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);	
			}
			prefs.setIconWidth(width);
			prefs.setDeferLargeImages(deferLoading.isSelected());
			prefs.setShowOnlyImages(showOnlyImages.isSelected());
			
			try {
				prefs.savePreferences();
			} 
			catch (IOException e) {
				JOptionPane.showMessageDialog(this, "Error saving preferences:"+e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			}
			setVisible(false);
			dispose();
		}


		else if (ae.getActionCommand().equals("cancel")) {
			setVisible(false);
			dispose();
		}

		else if (ae.getActionCommand().equals("find")) {
			JFileChooser chooser = new JFileChooser();
			int result = chooser.showOpenDialog(this);
			File file;
			if (result == JFileChooser.CANCEL_OPTION) {
				file = null;
				imageJPath.setText("");
			}
			else {
				file = chooser.getSelectedFile();
				imageJPath.setText(file.getAbsolutePath());
			}
		}
		
	}
}
